function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

import keyBy from 'lodash/keyBy';
import get from 'lodash/get';
import lowerCase from 'lodash/lowerCase';
import sum from 'lodash/sum';
import ExchangeError from '../base/errors/ExchangeError';
import BaseParser from '../base/BaseParser';
import { iso8601, milliseconds } from '../../utils/time';
import { precisionFromString } from '../../utils/number';
import { NULL_ID, ORDER_TYPE, MARKET_STATUS, ORDER_STATUSES, FEES } from './constants';

var BinanceParser =
/*#__PURE__*/
function (_BaseParser) {
  _inherits(BinanceParser, _BaseParser);

  function BinanceParser() {
    var _ref;

    var _temp, _this;

    _classCallCheck(this, BinanceParser);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _possibleConstructorReturn(_this, (_temp = _this = _possibleConstructorReturn(this, (_ref = BinanceParser.__proto__ || Object.getPrototypeOf(BinanceParser)).call.apply(_ref, [this].concat(args))), Object.defineProperty(_assertThisInitialized(_this), "parseTicker", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(ticker, market) {
        var symbol;
        var scopedMarket = market;
        var timestamp = parseFloat(ticker.closeTime);

        if (!timestamp) {
          timestamp = milliseconds();
        } // eslint-disable-next-line prefer-destructuring


        symbol = ticker.symbol;

        if (!market) {
          if (_this.exchange.marketsById[symbol]) {
            scopedMarket = _this.exchange.marketsById[symbol];
          }
        }

        if (market) {
          // eslint-disable-next-line prefer-destructuring
          symbol = scopedMarket.symbol;
        }

        return _extends({
          symbol: symbol,
          timestamp: timestamp,
          datetime: iso8601(timestamp),
          high: parseFloat(ticker.highPrice),
          low: parseFloat(ticker.lowPrice),
          bid: parseFloat(ticker.bidPrice),
          bidVolume: parseFloat(ticker.bidQty),
          ask: parseFloat(ticker.askPrice),
          askVolume: parseFloat(ticker.askQty),
          vwap: parseFloat(ticker.weightedAvgPrice),
          open: parseFloat(ticker.openPrice),
          close: parseFloat(ticker.prevClosePrice),
          first: undefined,
          last: parseFloat(ticker.lastPrice),
          change: parseFloat(ticker.priceChangePercent),
          percentage: undefined,
          average: undefined,
          baseVolume: parseFloat(ticker.volume),
          quoteVolume: parseFloat(ticker.quoteVolume)
        }, _this.infoField(ticker));
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseTickers", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(rawTickers) {
        var symbols = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
        var tickers = [];
        rawTickers.forEach(function (rawTicker) {
          tickers.push(_this.parseTicker(rawTicker));
        });
        var tickersBySymbol = keyBy(tickers, 'symbol'); // return all of them if no symbols were passed in the first argument

        if (!symbols) return tickersBySymbol; // otherwise filter by symbol

        var result = {};
        symbols.forEach(function (symbol) {
          if (tickersBySymbol[symbol]) {
            result[symbol] = tickersBySymbol[symbol];
          }
        });
        return result;
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOHLCV", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(ohlcv) {
        return [ohlcv[0], parseFloat(ohlcv[1]), parseFloat(ohlcv[2]), parseFloat(ohlcv[3]), parseFloat(ohlcv[4]), parseFloat(ohlcv[5])];
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOrderStatus", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(status) {
        if (status === ORDER_STATUSES.NEW) return 'open';
        if (status === ORDER_STATUSES.PARTIALLY_FILLED) return 'open';
        if (status === ORDER_STATUSES.FILLED) return 'closed';
        if (status === ORDER_STATUSES.CANCELED) return 'canceled';
        return lowerCase(status);
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOrder", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(order, market) {
        var symbol;
        var timestamp;
        var scopedMarket = market;
        var status = get(order, 'status');

        if (status) {
          status = _this.parseOrderStatus(status);
        }

        if (scopedMarket) {
          // eslint-disable-next-line prefer-destructuring
          symbol = scopedMarket.symbol;
        } else {
          var id = order.symbol;

          if (_this.exchange.marketsById[id]) {
            // eslint-disable-next-line prefer-destructuring
            scopedMarket = _this.exchange.marketsById[id]; // eslint-disable-next-line prefer-destructuring

            symbol = scopedMarket.symbol;
          }
        }

        if (order.time) {
          timestamp = order.time;
        } else if (order.transactTime) {
          timestamp = order.transactTime;
        } else {
          throw new ExchangeError("".concat(order.id, " malformed order: ").concat(JSON.stringify(order)));
        }

        var price = parseFloat(order.price);
        var amount = parseFloat(order.origQty);
        var filled = parseFloat(get(order, 'executedQty', 0.0));
        var remaining = Math.max(amount - filled, 0.0);

        var result = _extends({}, _this.infoField(order), {
          id: order.orderId.toString(),
          timestamp: timestamp,
          datetime: iso8601(timestamp),
          symbol: symbol,
          type: lowerCase(order.type),
          side: lowerCase(order.side),
          price: price,
          amount: amount,
          cost: price * amount,
          filled: filled,
          remaining: remaining,
          status: status,
          fee: undefined
        });

        return result;
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOrders", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(orders, market, since, limit) {
        var result = Object.values(orders).map(function (order) {
          return _this.parseOrder(order, market);
        });
        return _this.filterBySinceLimit(result, since, limit);
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "commonCurrencyCode", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(currency) {
        if (currency === 'BCC') return 'BCH';
        return currency;
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "currencyId", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(currency) {
        if (currency === 'BCH') return 'BCC';
        return currency;
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseMarkets", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(markets) {
        var result = [];
        markets.forEach(function (market) {
          var id = market.symbol;
          if (id === NULL_ID) return;
          var baseId = market.baseAsset;
          var quoteId = market.quoteAsset;

          var base = _this.commonCurrencyCode(baseId);

          var quote = _this.commonCurrencyCode(quoteId);

          var symbol = "".concat(base, "/").concat(quote);
          var filters = keyBy(market.filters, 'filterType');
          var precision = {
            base: market.baseAssetPrecision,
            quote: market.quotePrecision,
            amount: market.baseAssetPrecision,
            price: market.quotePrecision
          };
          var active = market.status === MARKET_STATUS.TRADING;
          var lot = -1 * Math.log10(precision.amount);

          var entry = _extends({}, FEES.trading, {
            id: id,
            symbol: symbol,
            base: base,
            quote: quote,
            baseId: baseId,
            quoteId: quoteId
          }, _this.infoField(market), {
            lot: lot,
            active: active,
            precision: precision,
            limits: {
              amount: {
                min: lot,
                max: undefined
              },
              price: {
                min: -1 * Math.log10(precision.price),
                max: undefined
              },
              cost: {
                min: lot,
                max: undefined
              }
            }
          });

          if (filters.PRICE_FILTER) {
            var filter = filters.PRICE_FILTER;
            entry.precision.price = precisionFromString(filter.tickSize);
            entry.limits.price = {
              min: parseFloat(filter.minPrice),
              max: parseFloat(filter.maxPrice)
            };
          }

          if (filters.LOT_SIZE) {
            var _filter = filters.LOT_SIZE;
            entry.precision.amount = precisionFromString(_filter.stepSize);
            entry.lot = parseFloat(_filter.stepSize);
            entry.limits.amount = {
              min: parseFloat(_filter.minQty),
              max: parseFloat(_filter.maxQty)
            };
          }

          if (filters.MIN_NOTIONAL) {
            entry.limits.cost.min = parseFloat(filters.MIN_NOTIONAL.minNotional);
          }

          result.push(entry);
        });
        return result;
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseTrade", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(trade, market) {
        var timestamp = trade.T || trade.time || trade.timestamp;
        var price = parseFloat(trade.p || trade.price);
        var amount = parseFloat(trade.q || trade.qty);
        var id = trade.a || trade.id || trade.aggId;
        var side;
        var order;
        var fee;

        if (trade.orderId) {
          order = trade.orderId.toString();
        }

        if (trade.m) {
          side = trade.m ? ORDER_TYPE.SELL : ORDER_TYPE.BUY; // this is reversed intentionally
        } else {
          side = trade.isBuyer ? ORDER_TYPE.BUY : ORDER_TYPE.SELL; // this is a true side
        }

        if (trade.commission) {
          fee = {
            cost: parseFloat(trade.commission),
            currency: _this.commonCurrencyCode(trade.commissionAsset)
          };
        }

        return _extends({}, _this.infoField(trade), {
          timestamp: timestamp,
          datetime: iso8601(timestamp),
          symbol: market.symbol,
          id: id,
          order: order,
          type: undefined,
          side: side,
          price: price,
          cost: price * amount,
          amount: amount,
          fee: fee
        });
      }
    }), _temp));
  }

  _createClass(BinanceParser, [{
    key: "parserBalances",
    value: function parserBalances(response) {
      var _this2 = this;

      var result = _extends({}, this.infoField(response));

      var balances = response.balances;
      balances.forEach(function (balance) {
        var asset = balance.asset;

        var currency = _this2.commonCurrencyCode(asset);

        var account = {
          free: parseFloat(balance.free),
          used: parseFloat(balance.locked),
          total: 0.0
        };
        account.total = sum([account.free, account.used]);
        result[currency] = account;
      });
      return this.parseBalance(result);
    }
  }]);

  return BinanceParser;
}(BaseParser);

export default BinanceParser;