function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

import get from 'lodash/get';
import BaseParser from '../base/BaseParser';
import { BASE_ASSETS, NON_BTC_TAKER } from './constants';
import { ORDER_TYPES, MARKET_STATUS, MAKER, TAKER } from '../../constants';
import { iso8601 as iso8601Fn, parse8601 } from '../../utils/time';
import { precisionFromString } from '../../utils/number';

var GdaxParser =
/*#__PURE__*/
function (_BaseParser) {
  _inherits(GdaxParser, _BaseParser);

  function GdaxParser() {
    var _ref;

    var _temp, _this;

    _classCallCheck(this, GdaxParser);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _possibleConstructorReturn(_this, (_temp = _this = _possibleConstructorReturn(this, (_ref = GdaxParser.__proto__ || Object.getPrototypeOf(GdaxParser)).call.apply(_ref, [this].concat(args))), Object.defineProperty(_assertThisInitialized(_this), "parseTrade", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(trade, market, marketsById) {
        var timestamp = trade.time ? parse8601(trade.time) : parse8601(trade.created_at);
        var iso8601 = timestamp && iso8601Fn(timestamp);
        var side = trade.side === ORDER_TYPES.LOWER.BUY ? ORDER_TYPES.LOWER.SELL : ORDER_TYPES.LOWER.BUY;
        var feeCost = parseFloat(get(trade, 'fill_fees', trade.fee));
        var feeRate;
        var feeCurrency;
        var symbol;
        var type;

        if (!market) {
          if (trade.product_id) {
            var marketId = trade.product_id;

            if (marketsById[marketId]) {
              // eslint-disable-next-line no-param-reassign
              market = marketsById[marketId];
            }
          }
        }

        if (market) {
          // eslint-disable-next-line prefer-destructuring
          symbol = market.symbol;
        }

        if (market) {
          feeCurrency = market.quote;

          if (trade.liquidity) {
            var rateType = trade.liquidity === 'T' ? TAKER : MAKER;
            feeRate = market[rateType];
          }
        }

        var fee = {
          cost: feeCost,
          currency: feeCurrency,
          rate: feeRate
        };
        var id = get(trade, 'trade_id');
        var orderId = get(trade, 'order_id');
        return _extends({
          id: id,
          order: orderId
        }, _this.infoField(trade), {
          timestamp: timestamp,
          datetime: iso8601,
          symbol: symbol,
          type: type,
          side: side,
          price: parseFloat(trade.price),
          amount: parseFloat(trade.size),
          fee: fee
        });
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOHLCV", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(ohlcv) {
        return [ohlcv[0] * 1000, ohlcv[3], ohlcv[2], ohlcv[1], ohlcv[4], ohlcv[5]];
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOrderStatus", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(status) {
        var statuses = {
          pending: 'open',
          active: 'open',
          open: 'open',
          done: 'closed',
          canceled: 'canceled'
        };
        return get(statuses, status);
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseOrder", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(order, _market) {
        var market = _market;

        if (!market) {
          if (_this.exchange.marketsById[order.product_id]) {
            market = _this.exchange.marketsById[order.product_id];
          }
        }

        var symbol = market && market.symbol;
        var timestamp = parse8601(order.created_at);

        var status = _this.parseOrderStatus(order.status);

        var price = parseFloat(get(order, 'price'));
        var amount = parseFloat(get(order, 'size') || get(order, 'funds') || get(order, 'specified_funds'));
        var filled = parseFloat(get(order, 'filled_size'));
        var cost = parseFloat(get(order, 'executed_value'));
        var fee = {
          cost: parseFloat(get(order, 'fill_fees')),
          currency: undefined,
          rate: undefined
        };
        var remaining;

        if (amount) {
          if (filled) {
            remaining = amount - filled;
          }
        }

        return _extends({
          id: order.id
        }, _this.infoField(order), {
          timestamp: timestamp,
          datetime: iso8601Fn(timestamp),
          status: status,
          symbol: symbol,
          type: order.type,
          side: order.side,
          price: price,
          cost: cost,
          amount: amount,
          filled: filled,
          remaining: remaining,
          fee: fee
        });
      }
    }), Object.defineProperty(_assertThisInitialized(_this), "parseMarkets", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(markets) {
        var result = [];
        var trading = _this.exchange.constructor.FEES.trading;
        markets.forEach(function (market) {
          var id = market.id,
              base = market.base_currency,
              quote = market.quote_currency;
          var symbol = "".concat(base, "/").concat(quote);
          var priceLimits = {
            min: market.quote_increment && parseFloat(market.quote_increment),
            max: undefined
          };
          var precision = {
            amount: 8,
            price: market.quote_increment && precisionFromString(market.quote_increment)
          };
          var taker = trading.taker;

          if (base === BASE_ASSETS.ETH || base === BASE_ASSETS.LTC) {
            taker = NON_BTC_TAKER;
          }

          var active = market.status === MARKET_STATUS.LOWER.ONLINE;
          result.push(_extends({}, trading, {
            id: id,
            symbol: symbol,
            base: base,
            quote: quote,
            precision: precision,
            limits: {
              amount: {
                min: market.base_min_size && parseFloat(market.base_min_size),
                max: market.base_max_size && parseFloat(market.base_max_size)
              },
              price: priceLimits,
              cost: {
                min: market.min_market_funds && parseFloat(market.min_market_funds),
                max: market.max_market_funds && parseFloat(market.max_market_funds)
              }
            },
            taker: taker,
            active: active
          }, _this.infoField(market)));
        });
        return result;
      }
    }), _temp));
  }

  _createClass(GdaxParser, [{
    key: "parseBalances",
    value: function parseBalances(balances) {
      var result = _extends({}, this.infoField(balances));

      balances.forEach(function (balance) {
        var currency = balance.currency;
        var account = {
          free: parseFloat(balance.available),
          used: parseFloat(balance.hold),
          total: parseFloat(balance.balance)
        };
        result[currency] = account;
      });
      return this.parseBalance(result);
    }
  }, {
    key: "parseTicker",
    value: function parseTicker(ticker, symbol) {
      var timestamp = ticker.time && parse8601(ticker.time);
      var datetime = timestamp && iso8601Fn(timestamp);
      var bid = ticker.bid ? parseFloat(ticker.bid) : null;
      var ask = ticker.ask ? parseFloat(ticker.ask) : null;
      return _extends({
        symbol: symbol,
        timestamp: timestamp,
        datetime: datetime,
        high: undefined,
        low: undefined,
        bid: bid,
        bidVolume: undefined,
        ask: ask,
        askVolume: undefined,
        vwap: undefined,
        open: undefined,
        close: undefined,
        first: undefined,
        last: parseFloat(ticker.price),
        change: undefined,
        percentage: undefined,
        average: undefined,
        baseVolume: parseFloat(ticker.volume),
        quoteVolume: undefined
      }, this.infoField(ticker));
    }
  }]);

  return GdaxParser;
}(BaseParser);

export default GdaxParser;