function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import crypto from 'crypto';
import sum from 'lodash/sum';
import BaseExchange from '../base/BaseExchange';
import ExchangeError from '../base/errors/ExchangeError';
import { ORDER_TYPES, ORDER_SIDES } from '../../constants';
import KucoinParser from './KucoinParser';
import { REQUIRED_CREDENTIALS, URLS, FEES, API, SIGNED_APIS } from './constants';

var Kucoin =
/*#__PURE__*/
function (_BaseExchange) {
  _inherits(Kucoin, _BaseExchange);

  function Kucoin() {
    _classCallCheck(this, Kucoin);

    return _possibleConstructorReturn(this, (Kucoin.__proto__ || Object.getPrototypeOf(Kucoin)).apply(this, arguments));
  }

  _createClass(Kucoin, [{
    key: "getSignature",
    value: function getSignature(path, queryString, nonce) {
      var strForSign = "".concat(path, "/").concat(nonce, "/").concat(queryString);
      var signatureStr = Buffer.from(strForSign).toString('base64');
      var signatureResult = crypto.createHmac('sha256', this.apiSecret).update(signatureStr).digest('hex');
      return signatureResult;
    }
  }, {
    key: "sign",
    value: function sign(options, path, queryString, nonce) {
      return {
        headers: {
          'KC-API-KEY': this.apiKey,
          'KC-API-NONCE': nonce,
          'KC-API-SIGNATURE': this.getSignature(path, queryString, nonce)
        }
      };
    }
  }, {
    key: "fetchUserInfo",
    value: function fetchUserInfo() {
      return this.signedRequest('get', '/user/info');
    }
  }, {
    key: "fetchMarkets",
    value: function () {
      var _fetchMarkets = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee() {
        var _this = this;

        var response, markets;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.api.public.get.marketOpenSymbols();

              case 2:
                response = _context.sent;
                markets = response.data;
                return _context.abrupt("return", markets.map(function (market) {
                  var id = market.symbol;
                  var base = market.coinType;
                  var quote = market.coinTypePair;
                  base = _this.commonCurrencyCode(base);
                  quote = _this.commonCurrencyCode(quote);
                  var symbol = "".concat(base, "/").concat(quote);
                  var precision = {
                    amount: 8,
                    price: 8
                  };
                  var active = market.trading;
                  return _extends({}, _this.constructor.FEES.trading, {
                    id: id,
                    symbol: symbol,
                    base: base,
                    quote: quote,
                    active: active,
                    info: market,
                    lot: Math.pow(10, -precision.amount),
                    precision: precision,
                    limits: {
                      amount: {
                        min: Math.pow(10, -precision.amount),
                        max: undefined
                      },
                      price: {
                        min: undefined,
                        max: undefined
                      }
                    }
                  });
                }));

              case 5:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function fetchMarkets() {
        return _fetchMarkets.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchCurrencies",
    value: function () {
      var _fetchCurrencies = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var _this2 = this;

        var params,
            _ref,
            currencies,
            result,
            _args2 = arguments;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                params = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : {};
                _context2.next = 3;
                return this.api.public.get.marketOpenCoins(params);

              case 3:
                _ref = _context2.sent;
                currencies = _ref.data;
                result = {};
                currencies.forEach(function (currency) {
                  var id = currency.coin;

                  var code = _this2.commonCurrencyCode(id);

                  var precision = currency.tradePrecision;
                  var deposit = currency.enableDeposit;
                  var withdraw = currency.enableWithdraw;
                  var active = deposit && withdraw;
                  result[code] = {
                    id: id,
                    code: code,
                    info: currency,
                    name: currency.name,
                    active: active,
                    status: 'ok',
                    fee: currency.withdrawFeeRate,
                    // todo: redesign
                    precision: precision,
                    limits: {
                      amount: {
                        min: Math.pow(10, -precision),
                        max: Math.pow(10, precision)
                      },
                      price: {
                        min: Math.pow(10, -precision),
                        max: Math.pow(10, precision)
                      },
                      cost: {
                        min: undefined,
                        max: undefined
                      },
                      withdraw: {
                        min: currency.withdrawMinAmount,
                        max: Math.pow(10, precision)
                      }
                    }
                  };
                });
                return _context2.abrupt("return", result);

              case 8:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      return function fetchCurrencies() {
        return _fetchCurrencies.apply(this, arguments);
      };
    }() // TODO: use symbols
    // eslint-disable-next-line no-unused-vars

  }, {
    key: "fetchTickers",
    value: function () {
      var _fetchTickers = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3() {
        var _this3 = this;

        var symbols,
            params,
            response,
            tickers,
            result,
            _args3 = arguments;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                symbols = _args3.length > 0 && _args3[0] !== undefined ? _args3[0] : undefined;
                params = _args3.length > 1 && _args3[1] !== undefined ? _args3[1] : {};
                _context3.next = 4;
                return this.api.public.get.marketOpenSymbols(params);

              case 4:
                response = _context3.sent;
                tickers = response.data;
                result = {};
                tickers.forEach(function (ticker) {
                  var parsed = _this3.parseTicker(ticker);

                  var symbol = parsed.symbol;
                  result[symbol] = ticker;
                });
                return _context3.abrupt("return", result);

              case 9:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      return function fetchTickers() {
        return _fetchTickers.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTicker",
    value: function () {
      var _fetchTicker = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee4(symbol) {
        var params,
            market,
            response,
            ticker,
            _args4 = arguments;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                params = _args4.length > 1 && _args4[1] !== undefined ? _args4[1] : {};
                _context4.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                _context4.next = 6;
                return this.api.public.get.openTick(_extends({
                  symbol: market.id
                }, params));

              case 6:
                response = _context4.sent;
                ticker = response.data;
                return _context4.abrupt("return", this.parseTicker(ticker, market));

              case 9:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      return function fetchTicker(_x) {
        return _fetchTicker.apply(this, arguments);
      };
    }()
  }, {
    key: "parseTrade",
    value: function parseTrade(trade) {
      var market = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var timestamp = trade[0];
      var side;

      if (trade[1] === ORDER_SIDES.BUY) {
        side = 'buy';
      } else if (trade[1] === ORDER_SIDES.SELL) {
        side = 'sell';
      }

      return {
        id: undefined,
        info: trade,
        timestamp: timestamp,
        datetime: this.iso8601(timestamp),
        symbol: market.symbol,
        type: ORDER_TYPES.LOWER.LIMIT,
        side: side,
        price: trade[2],
        amount: trade[3]
      };
    }
  }, {
    key: "fetchTrades",
    value: function () {
      var _fetchTrades = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5(symbol) {
        var since,
            limit,
            params,
            market,
            response,
            _args5 = arguments;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                since = _args5.length > 1 && _args5[1] !== undefined ? _args5[1] : undefined;
                limit = _args5.length > 2 && _args5[2] !== undefined ? _args5[2] : undefined;
                params = _args5.length > 3 && _args5[3] !== undefined ? _args5[3] : {};
                _context5.next = 5;
                return this.loadMarkets();

              case 5:
                market = this.market(symbol);
                _context5.next = 8;
                return this.api.public.get.openDealOrders(_extends({
                  symbol: market.id
                }, params));

              case 8:
                response = _context5.sent;
                return _context5.abrupt("return", this.parseTrades(response.data, market, since, limit));

              case 10:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      return function fetchTrades(_x2) {
        return _fetchTrades.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchBalance",
    value: function () {
      var _fetchBalance = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6() {
        var _this4 = this;

        var params,
            _ref2,
            balances,
            result,
            indexed,
            keys,
            _args6 = arguments;

        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                params = _args6.length > 0 && _args6[0] !== undefined ? _args6[0] : {};
                _context6.next = 3;
                return this.loadMarkets();

              case 3:
                _context6.next = 5;
                return this.api.private.get.accountBalance(_extends({
                  limit: 20,
                  // default 12, max 20
                  page: 1
                }, params));

              case 5:
                _ref2 = _context6.sent;
                balances = _ref2.data;
                result = {
                  info: balances
                };
                indexed = this.indexBy(balances, 'coinType');
                keys = Object.keys(indexed);
                keys.forEach(function (id) {
                  var currency = _this4.commonCurrencyCode(id);

                  var account = {};
                  var balance = indexed[id];
                  var used = parseFloat(balance.freezeBalance);
                  var free = parseFloat(balance.balance);
                  var total = sum([free, used]);
                  account.free = free;
                  account.used = used;
                  account.total = total;
                  result[currency] = account;
                });
                return _context6.abrupt("return", this.parseBalance(result));

              case 12:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      return function fetchBalance() {
        return _fetchBalance.apply(this, arguments);
      };
    }()
  }, {
    key: "createOrder",
    value: function () {
      var _createOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee7(symbol, type, side, amount) {
        var price,
            params,
            market,
            base,
            order,
            response,
            _args7 = arguments;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                price = _args7.length > 4 && _args7[4] !== undefined ? _args7[4] : undefined;
                params = _args7.length > 5 && _args7[5] !== undefined ? _args7[5] : {};

                if (!(type !== ORDER_TYPES.LOWER.LIMIT)) {
                  _context7.next = 4;
                  break;
                }

                throw new ExchangeError("".concat(this.id, " allows limit orders only"));

              case 4:
                _context7.next = 6;
                return this.loadMarkets();

              case 6:
                market = this.market(symbol);
                base = market.base;
                order = {
                  symbol: market.id,
                  type: side.toUpperCase(),
                  price: this.priceToPrecision(symbol, price),
                  amount: this.truncate(amount, this.currencies[base].precision)
                };
                _context7.next = 11;
                return this.api.private.postOrder(_extends({}, order, params));

              case 11:
                response = _context7.sent;
                return _context7.abrupt("return", {
                  info: response,
                  id: this.safeString(response.data, 'orderOid')
                });

              case 13:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      return function createOrder(_x3, _x4, _x5, _x6) {
        return _createOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "cancelOrder",
    value: function () {
      var _cancelOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee8(id) {
        var symbol,
            params,
            market,
            request,
            response,
            _args8 = arguments;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                symbol = _args8.length > 1 && _args8[1] !== undefined ? _args8[1] : undefined;
                params = _args8.length > 2 && _args8[2] !== undefined ? _args8[2] : {};

                if (symbol) {
                  _context8.next = 4;
                  break;
                }

                throw new ExchangeError("".concat(this.id, " cancelOrder requires symbol argument"));

              case 4:
                _context8.next = 6;
                return this.loadMarkets();

              case 6:
                market = this.market(symbol);
                request = {
                  symbol: market.id,
                  orderOid: id
                };

                if (!('type' in params)) {
                  _context8.next = 12;
                  break;
                }

                request.type = params.type.toUpperCase();
                _context8.next = 13;
                break;

              case 12:
                throw new ExchangeError("".concat(this.id, " cancelOrder requires type (BUY or SELL) param"));

              case 13:
                _context8.next = 15;
                return this.api.private.postCancelOrder(_extends({}, request, params));

              case 15:
                response = _context8.sent;
                return _context8.abrupt("return", response);

              case 17:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      return function cancelOrder(_x7) {
        return _cancelOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOpenOrders",
    value: function () {
      var _fetchOpenOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee9() {
        var symbol,
            since,
            limit,
            params,
            market,
            request,
            response,
            orders,
            result,
            _args9 = arguments;
        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                symbol = _args9.length > 0 && _args9[0] !== undefined ? _args9[0] : undefined;
                since = _args9.length > 1 && _args9[1] !== undefined ? _args9[1] : undefined;
                limit = _args9.length > 2 && _args9[2] !== undefined ? _args9[2] : undefined;
                params = _args9.length > 3 && _args9[3] !== undefined ? _args9[3] : {};

                if (symbol) {
                  _context9.next = 6;
                  break;
                }

                throw new ExchangeError("".concat(this.id, " fetchOpenOrders requires a symbol param"));

              case 6:
                _context9.next = 8;
                return this.loadMarkets();

              case 8:
                market = this.market(symbol);
                request = {
                  symbol: market.id
                };
                _context9.next = 12;
                return this.api.private.get.orderActiveMap(_extends({}, request, params));

              case 12:
                response = _context9.sent;
                orders = this.arrayConcat(response.data.SELL, response.data.BUY);
                result = [];
                orders.forEach(function (order) {
                  result.push(_extends({}, order, {
                    status: 'open'
                  }));
                });
                return _context9.abrupt("return", this.parseOrders(result, market, since, limit));

              case 17:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      return function fetchOpenOrders() {
        return _fetchOpenOrders.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchClosedOrders",
    value: function () {
      var _fetchClosedOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee10() {
        var symbol,
            since,
            limit,
            params,
            request,
            market,
            response,
            orders,
            result,
            _args10 = arguments;
        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                symbol = _args10.length > 0 && _args10[0] !== undefined ? _args10[0] : undefined;
                since = _args10.length > 1 && _args10[1] !== undefined ? _args10[1] : undefined;
                limit = _args10.length > 2 && _args10[2] !== undefined ? _args10[2] : undefined;
                params = _args10.length > 3 && _args10[3] !== undefined ? _args10[3] : {};
                request = {};
                _context10.next = 7;
                return this.loadMarkets();

              case 7:
                if (symbol) {
                  market = this.market(symbol);
                  request.symbol = market.id;
                }

                if (since) request.since = since;
                if (limit) request.limit = limit;
                _context10.next = 12;
                return this.api.private.get.orderDealt(_extends({}, request, params));

              case 12:
                response = _context10.sent;
                orders = response.data.datas;
                result = [];
                orders.forEach(function (order) {
                  result.push(_extends({}, order, {
                    status: 'closed'
                  }));
                });
                return _context10.abrupt("return", this.parseOrders(result, market, since, limit));

              case 17:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      return function fetchClosedOrders() {
        return _fetchClosedOrders.apply(this, arguments);
      };
    }()
  }, {
    key: "parseTradingViewOHLCVs",
    value: function parseTradingViewOHLCVs(ohlcvs) {
      var market = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var timeframe = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '1m';
      var since = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
      var limit = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;
      var result = [];

      for (var i = 0; i < ohlcvs.t.length; i += 1) {
        result.push([ohlcvs.t[i] * 1000, ohlcvs.o[i], ohlcvs.h[i], ohlcvs.l[i], ohlcvs.c[i], ohlcvs.v[i]]);
      }

      return this.parseOHLCVs(result, market, timeframe, since, limit);
    }
  }, {
    key: "fetchOHLCV",
    value: function () {
      var _fetchOHLCV = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee11(symbol) {
        var timeframe,
            since,
            limit,
            params,
            market,
            changedLimit,
            end,
            resolution,
            minutes,
            start,
            request,
            response,
            _args11 = arguments;
        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                timeframe = _args11.length > 1 && _args11[1] !== undefined ? _args11[1] : '1m';
                since = _args11.length > 2 && _args11[2] !== undefined ? _args11[2] : undefined;
                limit = _args11.length > 3 && _args11[3] !== undefined ? _args11[3] : undefined;
                params = _args11.length > 4 && _args11[4] !== undefined ? _args11[4] : {};
                _context11.next = 6;
                return this.loadMarkets();

              case 6:
                market = this.market(symbol);
                changedLimit = limit;
                end = this.seconds();
                resolution = this.timeframes[timeframe]; // convert 'resolution' to minutes in order to calculate 'from' later

                minutes = resolution;

                if (minutes === 'D') {
                  if (!limit) changedLimit = 30; // 30 days, 1 month

                  minutes = 1440;
                } else if (minutes === 'W') {
                  if (!changedLimit) changedLimit = 52; // 52 weeks, 1 year

                  minutes = 10080;
                } else if (!changedLimit) {
                  changedLimit = 1440;
                  minutes = 1440;
                  resolution = 'D';
                } // eslint-disable-next-line


                start = end - minutes * 60 * changedLimit;

                if (since) {
                  start = parseInt(since / 1000, 10);
                  end = this.sum(start, minutes * 60 * changedLimit);
                }

                request = {
                  symbol: market.id,
                  type: this.timeframes[timeframe],
                  resolution: resolution,
                  from: start,
                  to: end
                };
                _context11.next = 17;
                return this.api.kitchen.get.openChartHistory(_extends({}, request, params));

              case 17:
                response = _context11.sent;
                return _context11.abrupt("return", this.parseTradingViewOHLCVs(response, market, timeframe, since, changedLimit));

              case 19:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      return function fetchOHLCV(_x8) {
        return _fetchOHLCV.apply(this, arguments);
      };
    }()
  }]);

  return Kucoin;
}(BaseExchange);

Object.defineProperty(Kucoin, "Parser", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: KucoinParser
});
Object.defineProperty(Kucoin, "REQUIRED_CREDENTIALS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: REQUIRED_CREDENTIALS
});
Object.defineProperty(Kucoin, "URLS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: URLS
});
Object.defineProperty(Kucoin, "FEES", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: FEES
});
Object.defineProperty(Kucoin, "API", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: API
});
Object.defineProperty(Kucoin, "SIGNED_APIS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: SIGNED_APIS
});
export default Kucoin;