"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _get = _interopRequireDefault(require("lodash/get"));

var _upperCase = _interopRequireDefault(require("lodash/upperCase"));

var _qs = _interopRequireDefault(require("qs"));

var _BaseExchange2 = _interopRequireDefault(require("../base/BaseExchange"));

var _ExchangeError = _interopRequireDefault(require("../base/errors/ExchangeError"));

var _constants = require("../../constants");

var _validations = require("../../utils/validations");

var _time = require("../../utils/time");

var _debug = _interopRequireDefault(require("../../utils/debug"));

var _constants2 = require("./constants");

var _BinanceParser = _interopRequireDefault(require("./BinanceParser"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Binance =
/*#__PURE__*/
function (_BaseExchange) {
  _inherits(Binance, _BaseExchange);

  function Binance() {
    _classCallCheck(this, Binance);

    return _possibleConstructorReturn(this, (Binance.__proto__ || Object.getPrototypeOf(Binance)).apply(this, arguments));
  }

  _createClass(Binance, [{
    key: "getSignature",
    value: function getSignature(_ref) {
      var params = _ref.params,
          timestamp = _ref.timestamp;

      var strForSign = _qs.default.stringify(_extends({}, params, {
        timestamp: timestamp
      }));

      var signatureResult = _crypto.default.createHmac('sha256', this.apiSecret).update(strForSign).digest('hex');

      return signatureResult;
    }
  }, {
    key: "sign",
    value: function sign(_ref2) {
      var path = _ref2.path,
          params = _ref2.params;
      var nonce = new Date().getTime();

      if (this.verbose) {
        (0, _debug.default)("Signing with nonce: ".concat(nonce, ", path: ").concat(path, ", params: ").concat(params));
      }

      return {
        params: _extends({}, params, {
          timestamp: nonce,
          signature: this.getSignature({
            path: path,
            params: params,
            timestamp: nonce
          })
        }),
        headers: {
          'X-MBX-APIKEY': this.apiKey
        }
      };
    }
  }, {
    key: "loadTimeDifference",
    value: function () {
      var _loadTimeDifference = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee() {
        var before, response, after, sumOfDiff, averageOfDiff;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                before = (0, _time.milliseconds)();
                _context.next = 3;
                return this.api.public.get.time();

              case 3:
                response = _context.sent;
                after = (0, _time.milliseconds)();
                sumOfDiff = before + after;
                averageOfDiff = sumOfDiff / 2;
                this.timeDifference = averageOfDiff - response.serverTime;
                return _context.abrupt("return", this.timeDifference);

              case 9:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function loadTimeDifference() {
        return _loadTimeDifference.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchMarkets",
    value: function () {
      var _fetchMarkets = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var response, markets;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.api.public.get.exchangeInfo();

              case 2:
                response = _context2.sent;

                if (!this.adjustForTimeDifference) {
                  _context2.next = 6;
                  break;
                }

                _context2.next = 6;
                return this.loadTimeDifference();

              case 6:
                markets = response.symbols;
                return _context2.abrupt("return", this.parser.parseMarkets(markets));

              case 8:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      return function fetchMarkets() {
        return _fetchMarkets.apply(this, arguments);
      };
    }()
  }, {
    key: "calculateFee",
    value: function calculateFee() {
      var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          symbol = _ref3.symbol,
          side = _ref3.side,
          amount = _ref3.amount,
          price = _ref3.price,
          _ref3$takerOrMaker = _ref3.takerOrMaker,
          takerOrMaker = _ref3$takerOrMaker === void 0 ? _constants2.TAKER : _ref3$takerOrMaker;

      var key = 'quote';
      var market = this.markets[symbol];
      var rate = market[takerOrMaker];
      var cost = parseFloat(this.costToPrecision(symbol, amount * rate));

      if (side === _constants.ORDER_TYPES.LOWER.SELL) {
        cost *= price;
      } else {
        key = 'base';
      }

      return {
        type: takerOrMaker,
        currency: market[key],
        rate: rate,
        cost: parseFloat(this.feeToPrecision(symbol, cost))
      };
    }
  }, {
    key: "fetchBalance",
    value: function () {
      var _fetchBalance = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3() {
        var _ref4,
            _ref4$params,
            params,
            response,
            _args3 = arguments;

        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _ref4 = _args3.length > 0 && _args3[0] !== undefined ? _args3[0] : {}, _ref4$params = _ref4.params, params = _ref4$params === void 0 ? {} : _ref4$params;
                _context3.next = 3;
                return this.loadMarkets();

              case 3:
                _context3.next = 5;
                return this.api.private.get.account({
                  params: params
                });

              case 5:
                response = _context3.sent;
                return _context3.abrupt("return", this.parser.parserBalances(response));

              case 7:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      return function fetchBalance() {
        return _fetchBalance.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOrderBook",
    value: function () {
      var _fetchOrderBook = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee4() {
        var _ref5,
            symbol,
            _ref5$params,
            params,
            market,
            orderbook,
            _args4 = arguments;

        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _ref5 = _args4.length > 0 && _args4[0] !== undefined ? _args4[0] : {}, symbol = _ref5.symbol, _ref5$params = _ref5.params, params = _ref5$params === void 0 ? {} : _ref5$params;
                _context4.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                _context4.next = 6;
                return this.api.public.get.depth({
                  params: _extends({
                    symbol: market.id,
                    limit: 100
                  }, params)
                });

              case 6:
                orderbook = _context4.sent;
                return _context4.abrupt("return", this.parser.parseOrderBook(orderbook));

              case 8:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      return function fetchOrderBook() {
        return _fetchOrderBook.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTicker",
    value: function () {
      var _fetchTicker = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5() {
        var _ref6,
            symbol,
            _ref6$params,
            params,
            market,
            response,
            _args5 = arguments;

        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _ref6 = _args5.length > 0 && _args5[0] !== undefined ? _args5[0] : {}, symbol = _ref6.symbol, _ref6$params = _ref6.params, params = _ref6$params === void 0 ? {} : _ref6$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchTicker',
                  params: {
                    symbol: symbol
                  }
                });
                _context5.next = 4;
                return this.loadMarkets();

              case 4:
                market = this.market(symbol);
                _context5.next = 7;
                return this.api.public.get.ticker24Hr({
                  params: _extends({
                    symbol: market.id
                  }, params)
                });

              case 7:
                response = _context5.sent;
                return _context5.abrupt("return", this.parser.parseTicker(response, market, this.marketsById));

              case 9:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      return function fetchTicker() {
        return _fetchTicker.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchBidAsks",
    value: function () {
      var _fetchBidAsks = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6() {
        var _ref7,
            symbols,
            _ref7$params,
            params,
            rawTickers,
            _args6 = arguments;

        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _ref7 = _args6.length > 0 && _args6[0] !== undefined ? _args6[0] : {}, symbols = _ref7.symbols, _ref7$params = _ref7.params, params = _ref7$params === void 0 ? {} : _ref7$params;
                _context6.next = 3;
                return this.loadMarkets();

              case 3:
                _context6.next = 5;
                return this.api.public.get.tickerBookTicker({
                  params: params
                });

              case 5:
                rawTickers = _context6.sent;
                return _context6.abrupt("return", this.parser.parseTickers(rawTickers, symbols));

              case 7:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      return function fetchBidAsks() {
        return _fetchBidAsks.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTickers",
    value: function () {
      var _fetchTickers = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee7() {
        var _ref8,
            symbols,
            _ref8$params,
            params,
            rawTickers,
            _args7 = arguments;

        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _ref8 = _args7.length > 0 && _args7[0] !== undefined ? _args7[0] : {}, symbols = _ref8.symbols, _ref8$params = _ref8.params, params = _ref8$params === void 0 ? {} : _ref8$params;
                _context7.next = 3;
                return this.loadMarkets();

              case 3:
                _context7.next = 5;
                return this.api.public.get.ticker24Hr({
                  params: params
                });

              case 5:
                rawTickers = _context7.sent;
                return _context7.abrupt("return", this.parser.parseTickers(rawTickers, symbols));

              case 7:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      return function fetchTickers() {
        return _fetchTickers.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOHLCV",
    value: function () {
      var _fetchOHLCV = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee8() {
        var _ref9,
            symbol,
            _ref9$timeframe,
            timeframe,
            since,
            _ref9$limit,
            limit,
            _ref9$params,
            params,
            market,
            setupParams,
            response,
            _args8 = arguments;

        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _ref9 = _args8.length > 0 && _args8[0] !== undefined ? _args8[0] : {}, symbol = _ref9.symbol, _ref9$timeframe = _ref9.timeframe, timeframe = _ref9$timeframe === void 0 ? _constants2.TIMEFRAMES['1m'] : _ref9$timeframe, since = _ref9.since, _ref9$limit = _ref9.limit, limit = _ref9$limit === void 0 ? _constants2.DEFAULT_OHLCV_LIMIT : _ref9$limit, _ref9$params = _ref9.params, params = _ref9$params === void 0 ? {} : _ref9$params;
                _context8.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                setupParams = {
                  symbol: market.id,
                  interval: _constants2.TIMEFRAMES[timeframe],
                  limit: limit // default == max == 500

                };

                if (since) {
                  setupParams.startTime = since;
                }

                _context8.next = 8;
                return this.api.public.get.klines({
                  params: _extends({}, setupParams, params)
                });

              case 8:
                response = _context8.sent;
                return _context8.abrupt("return", this.parser.parseOHLCVs(response, market, timeframe, since, limit));

              case 10:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      return function fetchOHLCV() {
        return _fetchOHLCV.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTrades",
    value: function () {
      var _fetchTrades = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee9() {
        var _ref10,
            symbol,
            since,
            limit,
            _ref10$params,
            params,
            market,
            setupParams,
            response,
            _args9 = arguments;

        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _ref10 = _args9.length > 0 && _args9[0] !== undefined ? _args9[0] : {}, symbol = _ref10.symbol, since = _ref10.since, limit = _ref10.limit, _ref10$params = _ref10.params, params = _ref10$params === void 0 ? {} : _ref10$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchTrades',
                  params: {
                    symbol: symbol
                  },
                  error: _ExchangeError.default
                });
                _context9.next = 4;
                return this.loadMarkets();

              case 4:
                market = this.market(symbol);
                setupParams = {
                  symbol: market.id,
                  limit: limit
                };

                if (since) {
                  setupParams.startTime = since;
                  setupParams.endTime = since + 3600000;
                } // 'fromId': 123,    // ID to get aggregate trades from INCLUSIVE.
                // 'startTime': 456, // Timestamp in ms to get aggregate trades from INCLUSIVE.
                // 'endTime': 789,   // Timestamp in ms to get aggregate trades until INCLUSIVE.
                // 'limit': 500,     // default = maximum = 500


                _context9.next = 9;
                return this.api.public.get.aggTrades({
                  symbol: market.id,
                  params: _extends({}, setupParams, params)
                });

              case 9:
                response = _context9.sent;
                return _context9.abrupt("return", this.parser.parseTrades(response, market, since, limit));

              case 11:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      return function fetchTrades() {
        return _fetchTrades.apply(this, arguments);
      };
    }()
  }, {
    key: "createOrder",
    value: function () {
      var _createOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee10() {
        var _ref11,
            symbol,
            type,
            side,
            amount,
            price,
            _ref11$params,
            params,
            market,
            order,
            response,
            _args10 = arguments;

        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                _ref11 = _args10.length > 0 && _args10[0] !== undefined ? _args10[0] : {}, symbol = _ref11.symbol, type = _ref11.type, side = _ref11.side, amount = _ref11.amount, price = _ref11.price, _ref11$params = _ref11.params, params = _ref11$params === void 0 ? {} : _ref11$params;
                _context10.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                order = {
                  symbol: market.id,
                  quantity: this.amountToString(symbol, amount),
                  type: (0, _upperCase.default)(type),
                  side: (0, _upperCase.default)(side)
                };

                if (type === _constants.ORDER_TYPES.LOWER.LIMIT) {
                  order.price = this.priceToPrecision(symbol, price);
                  order.timeInForce = _constants2.TIME_IN_FORCE.GTC;
                }

                _context10.next = 8;
                return this.api.private.post.order({
                  data: _extends({}, order, params)
                });

              case 8:
                response = _context10.sent;
                return _context10.abrupt("return", this.parser.parseOrder(response, this.marketsById));

              case 10:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      return function createOrder() {
        return _createOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOrder",
    value: function () {
      var _fetchOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee11() {
        var _ref12,
            id,
            symbol,
            _ref12$params,
            params,
            market,
            response,
            _args11 = arguments;

        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                _ref12 = _args11.length > 0 && _args11[0] !== undefined ? _args11[0] : {}, id = _ref12.id, symbol = _ref12.symbol, _ref12$params = _ref12.params, params = _ref12$params === void 0 ? {} : _ref12$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchOrder',
                  params: {
                    symbol: symbol,
                    id: id
                  },
                  error: _ExchangeError.default
                });
                _context11.next = 4;
                return this.loadMarkets();

              case 4:
                market = this.market(symbol);
                _context11.next = 7;
                return this.api.private.get.order({
                  params: _extends({
                    symbol: market.id,
                    orderId: parseInt(id, 10)
                  }, params)
                });

              case 7:
                response = _context11.sent;
                return _context11.abrupt("return", this.parser.parseOrder(response, market));

              case 9:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      return function fetchOrder() {
        return _fetchOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOrders",
    value: function () {
      var _fetchOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee12() {
        var _ref13,
            symbol,
            since,
            limit,
            _ref13$params,
            params,
            market,
            response,
            _args12 = arguments;

        return regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                _ref13 = _args12.length > 0 && _args12[0] !== undefined ? _args12[0] : {}, symbol = _ref13.symbol, since = _ref13.since, limit = _ref13.limit, _ref13$params = _ref13.params, params = _ref13$params === void 0 ? {} : _ref13$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchOrders',
                  params: {
                    symbol: symbol
                  },
                  error: _ExchangeError.default
                });
                _context12.next = 4;
                return this.loadMarkets();

              case 4:
                market = this.market(symbol);
                _context12.next = 7;
                return this.api.private.get.allOrders({
                  symbol: market.id,
                  params: _extends({
                    limit: limit
                  }, params)
                });

              case 7:
                response = _context12.sent;
                return _context12.abrupt("return", this.parser.parseOrders(response, market, since, limit));

              case 9:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      return function fetchOrders() {
        return _fetchOrders.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOpenOrders",
    value: function () {
      var _fetchOpenOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee13() {
        var _ref14,
            symbol,
            since,
            limit,
            _ref14$params,
            params,
            market,
            setupParams,
            response,
            _args13 = arguments;

        return regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                _ref14 = _args13.length > 0 && _args13[0] !== undefined ? _args13[0] : {}, symbol = _ref14.symbol, since = _ref14.since, limit = _ref14.limit, _ref14$params = _ref14.params, params = _ref14$params === void 0 ? {} : _ref14$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchOpenOrders',
                  params: {
                    symbol: symbol
                  },
                  error: _ExchangeError.default
                });
                _context13.next = 4;
                return this.loadMarkets();

              case 4:
                setupParams = {};

                if (symbol) {
                  market = this.market(symbol);
                  setupParams.symbol = market.id;
                }

                _context13.next = 8;
                return this.api.private.get.openOrders(_extends({}, setupParams, {
                  params: params
                }));

              case 8:
                response = _context13.sent;
                return _context13.abrupt("return", this.parser.parseOrders(response, market, since, limit));

              case 10:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      return function fetchOpenOrders() {
        return _fetchOpenOrders.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchMyTrades",
    value: function () {
      var _fetchMyTrades = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee14() {
        var _ref15,
            symbol,
            since,
            limit,
            _ref15$params,
            params,
            market,
            response,
            _args14 = arguments;

        return regeneratorRuntime.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                _ref15 = _args14.length > 0 && _args14[0] !== undefined ? _args14[0] : {}, symbol = _ref15.symbol, since = _ref15.since, limit = _ref15.limit, _ref15$params = _ref15.params, params = _ref15$params === void 0 ? {} : _ref15$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchMyTrades',
                  params: {
                    symbol: symbol
                  },
                  error: _ExchangeError.default
                });
                _context14.next = 4;
                return this.loadMarkets();

              case 4:
                market = this.market(symbol);
                _context14.next = 7;
                return this.api.private.get.myTrades({
                  params: _extends({
                    symbol: market.id,
                    limit: limit
                  }, params)
                });

              case 7:
                response = _context14.sent;
                return _context14.abrupt("return", this.parser.parseTrades(response, market, since, limit));

              case 9:
              case "end":
                return _context14.stop();
            }
          }
        }, _callee14, this);
      }));

      return function fetchMyTrades() {
        return _fetchMyTrades.apply(this, arguments);
      };
    }()
  }, {
    key: "withdraw",
    value: function () {
      var _withdraw = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee15() {
        var _ref16,
            currency,
            amount,
            address,
            tag,
            _ref16$params,
            params,
            name,
            request,
            response,
            _args15 = arguments;

        return regeneratorRuntime.wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                _ref16 = _args15.length > 0 && _args15[0] !== undefined ? _args15[0] : {}, currency = _ref16.currency, amount = _ref16.amount, address = _ref16.address, tag = _ref16.tag, _ref16$params = _ref16.params, params = _ref16$params === void 0 ? {} : _ref16$params;
                (0, _validations.validateRequiredParams)({
                  name: 'fetchMyTrades',
                  params: {
                    address: address,
                    amount: amount,
                    currency: currency
                  },
                  error: _ExchangeError.default
                });
                name = address.slice(0, 20);
                request = {
                  asset: this.parser.currencyId(currency),
                  address: address,
                  amount: parseFloat(amount),
                  name: name
                };

                if (tag) {
                  request.addressTag = tag;
                }

                _context15.next = 7;
                return this.api.wapi.post.withdraw({
                  data: _extends({}, request, params)
                });

              case 7:
                response = _context15.sent;
                return _context15.abrupt("return", _extends({}, this.parser.infoField(response), {
                  id: (0, _get.default)(response, 'id')
                }));

              case 9:
              case "end":
                return _context15.stop();
            }
          }
        }, _callee15, this);
      }));

      return function withdraw() {
        return _withdraw.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchDepositAddress",
    value: function () {
      var _fetchDepositAddress = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee16() {
        var _ref17,
            currency,
            _ref17$params,
            params,
            response,
            address,
            tag,
            _args16 = arguments;

        return regeneratorRuntime.wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                _ref17 = _args16.length > 0 && _args16[0] !== undefined ? _args16[0] : {}, currency = _ref17.currency, _ref17$params = _ref17.params, params = _ref17$params === void 0 ? {} : _ref17$params;
                _context16.next = 3;
                return this.api.wapi.get.depositAddress({
                  asset: this.parser.currencyId(currency),
                  params: params
                });

              case 3:
                response = _context16.sent;

                if (!response.success) {
                  _context16.next = 8;
                  break;
                }

                address = (0, _get.default)(response, 'address');
                tag = (0, _get.default)(response, 'addressTag');
                return _context16.abrupt("return", _extends({
                  currency: currency,
                  address: address,
                  tag: tag,
                  status: 'ok'
                }, this.parser.infoField(response)));

              case 8:
                throw new _ExchangeError.default("Binance fetchDepositAddress failed: ".concat(this.last_http_response));

              case 9:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this);
      }));

      return function fetchDepositAddress() {
        return _fetchDepositAddress.apply(this, arguments);
      };
    }()
  }]);

  return Binance;
}(_BaseExchange2.default);

Object.defineProperty(Binance, "Parser", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _BinanceParser.default
});
Object.defineProperty(Binance, "REQUIRED_CREDENTIALS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.REQUIRED_CREDENTIALS
});
Object.defineProperty(Binance, "URLS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.URLS
});
Object.defineProperty(Binance, "FEES", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.FEES
});
Object.defineProperty(Binance, "API", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.API
});
Object.defineProperty(Binance, "SIGNED_APIS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.SIGNED_APIS
});
var _default = Binance;
exports.default = _default;