"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _sum = _interopRequireDefault(require("lodash/sum"));

var _get = _interopRequireDefault(require("lodash/get"));

var _upperCase = _interopRequireDefault(require("lodash/upperCase"));

var _BaseExchange2 = _interopRequireDefault(require("../base/BaseExchange"));

var _NotSupported = _interopRequireDefault(require("../base/errors/NotSupported"));

var _ExchangeError = _interopRequireDefault(require("../base/errors/ExchangeError"));

var _constants = require("../../constants");

var _constants2 = require("./constants");

var _GdaxParser = _interopRequireDefault(require("./GdaxParser"));

var _time = require("../../utils/time");

var _debug = _interopRequireDefault(require("../../utils/debug"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Gdax =
/*#__PURE__*/
function (_BaseExchange) {
  _inherits(Gdax, _BaseExchange);

  function Gdax() {
    _classCallCheck(this, Gdax);

    return _possibleConstructorReturn(this, (Gdax.__proto__ || Object.getPrototypeOf(Gdax)).apply(this, arguments));
  }

  _createClass(Gdax, [{
    key: "getSignature",
    value: function getSignature(_ref) {
      var nonce = _ref.nonce,
          method = _ref.method,
          url = _ref.url,
          data = _ref.data;
      var body = data ? JSON.stringify(data) : '';
      var strForSign = nonce + (0, _upperCase.default)(method) + url + body;
      var key = Buffer.from(this.apiSecret, 'base64');
      return _crypto.default.createHmac('sha256', key).update(strForSign).digest('base64');
    }
  }, {
    key: "sign",
    value: function sign(_ref2) {
      var method = _ref2.method,
          url = _ref2.url,
          data = _ref2.data;
      var nonce = Date.now() / 1000;

      if (this.verbose) {
        (0, _debug.default)("Signing with ".concat(nonce, ", method: ").concat(method, ", path: ").concat(url, ", data: ").concat(data));
      }

      return {
        headers: {
          'CB-ACCESS-KEY': this.apiKey,
          'CB-ACCESS-SIGN': this.getSignature({
            method: method,
            url: url,
            data: data,
            nonce: nonce
          }),
          'CB-ACCESS-TIMESTAMP': nonce,
          'CB-ACCESS-PASSPHRASE': this.password
        }
      };
    }
  }, {
    key: "fetchMarkets",
    value: function () {
      var _fetchMarkets = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee() {
        var markets;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.api.public.get.products();

              case 2:
                markets = _context.sent;
                return _context.abrupt("return", this.parser.parseMarkets(markets));

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function fetchMarkets() {
        return _fetchMarkets.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchBalance",
    value: function () {
      var _fetchBalance = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var balances;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.loadMarkets();

              case 2:
                _context2.next = 4;
                return this.api.private.get.accounts();

              case 4:
                balances = _context2.sent;
                return _context2.abrupt("return", this.parser.parseBalances(balances));

              case 6:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      return function fetchBalance() {
        return _fetchBalance.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOrderBook",
    value: function () {
      var _fetchOrderBook = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3() {
        var _ref3,
            symbol,
            _ref3$level,
            level,
            _ref3$params,
            params,
            orderbook,
            _args3 = arguments;

        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _ref3 = _args3.length > 0 && _args3[0] !== undefined ? _args3[0] : {}, symbol = _ref3.symbol, _ref3$level = _ref3.level, level = _ref3$level === void 0 ? 2 : _ref3$level, _ref3$params = _ref3.params, params = _ref3$params === void 0 ? {} : _ref3$params;
                _context3.next = 3;
                return this.loadMarkets();

              case 3:
                _context3.next = 5;
                return this.api.public.get.productsIdBook({
                  id: this.marketId(symbol),
                  params: _extends({
                    level: level
                  }, params)
                });

              case 5:
                orderbook = _context3.sent;
                return _context3.abrupt("return", this.parser.parseOrderBook(orderbook));

              case 7:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      return function fetchOrderBook() {
        return _fetchOrderBook.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTicker",
    value: function () {
      var _fetchTicker = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee4() {
        var _ref4,
            symbol,
            _ref4$params,
            params,
            market,
            ticker,
            _args4 = arguments;

        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _ref4 = _args4.length > 0 && _args4[0] !== undefined ? _args4[0] : {}, symbol = _ref4.symbol, _ref4$params = _ref4.params, params = _ref4$params === void 0 ? {} : _ref4$params;
                _context4.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                _context4.next = 6;
                return this.api.public.get.productsIdTicker({
                  id: market.id,
                  params: params
                });

              case 6:
                ticker = _context4.sent;
                return _context4.abrupt("return", this.parser.parseTicker(ticker, symbol));

              case 8:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      return function fetchTicker() {
        return _fetchTicker.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchMyTrades",
    value: function () {
      var _fetchMyTrades = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5() {
        var _ref5,
            symbol,
            since,
            limit,
            _ref5$params,
            params,
            market,
            request,
            response,
            _args5 = arguments;

        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _ref5 = _args5.length > 0 && _args5[0] !== undefined ? _args5[0] : {}, symbol = _ref5.symbol, since = _ref5.since, limit = _ref5.limit, _ref5$params = _ref5.params, params = _ref5$params === void 0 ? {} : _ref5$params;
                _context5.next = 3;
                return this.loadMarkets();

              case 3:
                request = {};

                if (symbol) {
                  market = this.market(symbol);
                  request.product_id = market.id;
                }

                _context5.next = 7;
                return this.api.private.get.fills(_extends({}, request, {
                  params: _extends({
                    limit: limit
                  }, params)
                }));

              case 7:
                response = _context5.sent;
                return _context5.abrupt("return", this.parser.parseTrades(response, market, since, limit, this.marketsById));

              case 9:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      return function fetchMyTrades() {
        return _fetchMyTrades.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTrades",
    value: function () {
      var _fetchTrades = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6() {
        var _ref6,
            symbol,
            since,
            limit,
            _ref6$params,
            params,
            market,
            response,
            _args6 = arguments;

        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _ref6 = _args6.length > 0 && _args6[0] !== undefined ? _args6[0] : {}, symbol = _ref6.symbol, since = _ref6.since, limit = _ref6.limit, _ref6$params = _ref6.params, params = _ref6$params === void 0 ? {} : _ref6$params;
                _context6.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                _context6.next = 6;
                return this.api.public.get.productsIdTrades({
                  id: market.id,
                  // fixes issue #2
                  params: params
                });

              case 6:
                response = _context6.sent;
                return _context6.abrupt("return", this.parser.parseTrades(response, market, since, limit, this.marketsById));

              case 8:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      return function fetchTrades() {
        return _fetchTrades.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOHLCV",
    value: function () {
      var _fetchOHLCV = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee7() {
        var _ref7,
            symbol,
            _ref7$timeframe,
            timeframe,
            since,
            _ref7$limit,
            limit,
            _ref7$params,
            params,
            market,
            granularity,
            setupParams,
            response,
            _args7 = arguments;

        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _ref7 = _args7.length > 0 && _args7[0] !== undefined ? _args7[0] : {}, symbol = _ref7.symbol, _ref7$timeframe = _ref7.timeframe, timeframe = _ref7$timeframe === void 0 ? '1m' : _ref7$timeframe, since = _ref7.since, _ref7$limit = _ref7.limit, limit = _ref7$limit === void 0 ? 350 : _ref7$limit, _ref7$params = _ref7.params, params = _ref7$params === void 0 ? {} : _ref7$params;
                _context7.next = 3;
                return this.loadMarkets();

              case 3:
                market = this.market(symbol);
                granularity = _constants2.TIMEFRAMES[timeframe];
                setupParams = {
                  granularity: granularity,
                  // https://docs.gdax.com/#get-historic-rates
                  limit: limit // max = 350

                };

                if (since) {
                  setupParams.start = (0, _time.ymdhms)(since);
                  setupParams.end = (0, _time.ymdhms)((0, _sum.default)([limit * granularity * 1000, since]));
                }

                _context7.next = 9;
                return this.api.public.get.productsIdCandles({
                  id: market.id,
                  params: _extends({}, setupParams, {
                    granularity: granularity
                  }, params)
                });

              case 9:
                response = _context7.sent;
                return _context7.abrupt("return", this.parser.parseOHLCVs(response, market, timeframe, since, limit));

              case 11:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      return function fetchOHLCV() {
        return _fetchOHLCV.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchTime",
    value: function () {
      var _fetchTime = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee8() {
        var response;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _context8.next = 2;
                return this.api.public.get.time();

              case 2:
                response = _context8.sent;
                return _context8.abrupt("return", (0, _time.parse8601)(response.iso));

              case 4:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      return function fetchTime() {
        return _fetchTime.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOrder",
    value: function () {
      var _fetchOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee9() {
        var _ref8,
            id,
            _ref8$params,
            params,
            response,
            _args9 = arguments;

        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _ref8 = _args9.length > 0 && _args9[0] !== undefined ? _args9[0] : {}, id = _ref8.id, _ref8$params = _ref8.params, params = _ref8$params === void 0 ? {} : _ref8$params;
                _context9.next = 3;
                return this.loadMarkets();

              case 3:
                _context9.next = 5;
                return this.api.private.get.ordersId({
                  id: id,
                  params: params
                });

              case 5:
                response = _context9.sent;
                return _context9.abrupt("return", this.parser.parseOrder(response));

              case 7:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      return function fetchOrder() {
        return _fetchOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOrders",
    value: function () {
      var _fetchOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee10() {
        var _ref9,
            symbol,
            since,
            limit,
            _ref9$params,
            params,
            request,
            market,
            response,
            _args10 = arguments;

        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                _ref9 = _args10.length > 0 && _args10[0] !== undefined ? _args10[0] : {}, symbol = _ref9.symbol, since = _ref9.since, limit = _ref9.limit, _ref9$params = _ref9.params, params = _ref9$params === void 0 ? {} : _ref9$params;
                _context10.next = 3;
                return this.loadMarkets();

              case 3:
                request = {};

                if (symbol) {
                  market = this.market(symbol);
                  request.product_id = market.id;
                }

                _context10.next = 7;
                return this.api.private.get.orders(_extends({}, request, {
                  params: _extends({}, params, {
                    status: _constants.ORDER_STATUSES.LOWER.ALL
                  })
                }));

              case 7:
                response = _context10.sent;
                return _context10.abrupt("return", this.parser.parseOrders(response, market, since, limit));

              case 9:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      return function fetchOrders() {
        return _fetchOrders.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchOpenOrders",
    value: function () {
      var _fetchOpenOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee11() {
        var _ref10,
            symbol,
            since,
            limit,
            _ref10$params,
            params,
            request,
            market,
            response,
            _args11 = arguments;

        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                _ref10 = _args11.length > 0 && _args11[0] !== undefined ? _args11[0] : {}, symbol = _ref10.symbol, since = _ref10.since, limit = _ref10.limit, _ref10$params = _ref10.params, params = _ref10$params === void 0 ? {} : _ref10$params;
                _context11.next = 3;
                return this.loadMarkets();

              case 3:
                request = {};

                if (symbol) {
                  market = this.market(symbol);
                  request.product_id = market.id;
                }

                _context11.next = 7;
                return this.api.private.get.orders(_extends({}, request, {
                  params: params
                }));

              case 7:
                response = _context11.sent;
                return _context11.abrupt("return", this.parser.parseOrders(response, market, since, limit));

              case 9:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      return function fetchOpenOrders() {
        return _fetchOpenOrders.apply(this, arguments);
      };
    }()
  }, {
    key: "fetchClosedOrders",
    value: function () {
      var _fetchClosedOrders = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee12() {
        var _ref11,
            symbol,
            since,
            limit,
            _ref11$params,
            params,
            request,
            market,
            response,
            _args12 = arguments;

        return regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                _ref11 = _args12.length > 0 && _args12[0] !== undefined ? _args12[0] : {}, symbol = _ref11.symbol, since = _ref11.since, limit = _ref11.limit, _ref11$params = _ref11.params, params = _ref11$params === void 0 ? {} : _ref11$params;
                _context12.next = 3;
                return this.loadMarkets();

              case 3:
                request = {};

                if (symbol) {
                  market = this.market(symbol);
                  request.product_id = market.id;
                }

                _context12.next = 7;
                return this.api.private.get.orders(_extends({}, request, {
                  params: _extends({}, params, {
                    status: _constants.ORDER_STATUSES.LOWER.DONE
                  })
                }));

              case 7:
                response = _context12.sent;
                return _context12.abrupt("return", this.parser.parseOrders(response, market, since, limit));

              case 9:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      return function fetchClosedOrders() {
        return _fetchClosedOrders.apply(this, arguments);
      };
    }() // TODO: Add more order types

  }, {
    key: "createOrder",
    value: function () {
      var _createOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee13() {
        var _ref12,
            market,
            type,
            side,
            amount,
            price,
            _ref12$params,
            params,
            order,
            response,
            _args13 = arguments;

        return regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                _ref12 = _args13.length > 0 && _args13[0] !== undefined ? _args13[0] : {}, market = _ref12.market, type = _ref12.type, side = _ref12.side, amount = _ref12.amount, price = _ref12.price, _ref12$params = _ref12.params, params = _ref12$params === void 0 ? {} : _ref12$params;
                _context13.next = 3;
                return this.loadMarkets();

              case 3:
                // let oid = this.nonce ().toString ();
                order = _extends({
                  product_id: this.marketId(market),
                  side: side,
                  size: amount,
                  type: type
                }, params);

                if (type === _constants.ORDER_TYPES.LOWER.LIMIT) {
                  order.price = price;
                }

                _context13.next = 7;
                return this.api.private.post.orders({
                  data: order
                });

              case 7:
                response = _context13.sent;
                return _context13.abrupt("return", {
                  info: response,
                  id: response.id
                });

              case 9:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      return function createOrder() {
        return _createOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "cancelOrder",
    value: function () {
      var _cancelOrder = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee14(_ref13) {
        var id, response;
        return regeneratorRuntime.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                id = _ref13.id;
                _context14.next = 3;
                return this.loadMarkets();

              case 3:
                _context14.next = 5;
                return this.api.private.delete.ordersId({
                  id: id
                });

              case 5:
                response = _context14.sent;
                return _context14.abrupt("return", response);

              case 7:
              case "end":
                return _context14.stop();
            }
          }
        }, _callee14, this);
      }));

      return function cancelOrder(_x) {
        return _cancelOrder.apply(this, arguments);
      };
    }()
  }, {
    key: "getPaymentMethods",
    value: function () {
      var _getPaymentMethods = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee15() {
        var response;
        return regeneratorRuntime.wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                _context15.next = 2;
                return this.api.private.get.paymentMethods();

              case 2:
                response = _context15.sent;
                return _context15.abrupt("return", response);

              case 4:
              case "end":
                return _context15.stop();
            }
          }
        }, _callee15, this);
      }));

      return function getPaymentMethods() {
        return _getPaymentMethods.apply(this, arguments);
      };
    }()
  }, {
    key: "deposit",
    value: function () {
      var _deposit = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee16() {
        var _ref14,
            currency,
            amount,
            _ref14$params,
            params,
            request,
            method,
            methodFunction,
            response,
            _args16 = arguments;

        return regeneratorRuntime.wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                _ref14 = _args16.length > 0 && _args16[0] !== undefined ? _args16[0] : {}, currency = _ref14.currency, amount = _ref14.amount, _ref14$params = _ref14.params, params = _ref14$params === void 0 ? {} : _ref14$params;
                _context16.next = 3;
                return this.loadMarkets();

              case 3:
                request = {
                  currency: currency,
                  amount: amount
                };
                method = 'api.private.post.deposits';

                if (!('payment_method_id' in params)) {
                  _context16.next = 9;
                  break;
                }

                // deposit from a payment_method, like a bank account
                method += 'PaymentMethod';
                _context16.next = 14;
                break;

              case 9:
                if (!('coinbase_account_id' in params)) {
                  _context16.next = 13;
                  break;
                }

                // deposit into GDAX account from a Coinbase account
                method += 'CoinbaseAccount';
                _context16.next = 14;
                break;

              case 13:
                throw new _NotSupported.default('Gdax deposit() requires one of `coinbase_account_id` or `payment_method_id` extra params');

              case 14:
                methodFunction = (0, _get.default)(this, method);
                _context16.next = 17;
                return methodFunction(_extends({}, request, params));

              case 17:
                response = _context16.sent;

                if (response) {
                  _context16.next = 20;
                  break;
                }

                throw new _ExchangeError.default("Gdax deposit() error: ".concat(JSON.stringify(response)));

              case 20:
                return _context16.abrupt("return", {
                  info: response,
                  id: response.id
                });

              case 21:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this);
      }));

      return function deposit() {
        return _deposit.apply(this, arguments);
      };
    }()
  }, {
    key: "withdraw",
    value: function () {
      var _withdraw = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee17() {
        var _ref15,
            currency,
            amount,
            address,
            _ref15$params,
            params,
            data,
            method,
            methodFunction,
            response,
            _args17 = arguments;

        return regeneratorRuntime.wrap(function _callee17$(_context17) {
          while (1) {
            switch (_context17.prev = _context17.next) {
              case 0:
                _ref15 = _args17.length > 0 && _args17[0] !== undefined ? _args17[0] : {}, currency = _ref15.currency, amount = _ref15.amount, address = _ref15.address, _ref15$params = _ref15.params, params = _ref15$params === void 0 ? {} : _ref15$params;
                _context17.next = 3;
                return this.loadMarkets();

              case 3:
                data = {
                  currency: currency,
                  amount: amount
                };
                method = 'api.private.post.withdrawals';

                if ('payment_method_id' in params) {
                  method += 'PaymentMethod';
                } else if ('coinbase_account_id' in params) {
                  method += 'CoinbaseAccount';
                } else {
                  method += 'Crypto';
                  data.crypto_address = address;
                }

                methodFunction = (0, _get.default)(this, method);
                _context17.next = 9;
                return methodFunction({
                  data: data,
                  params: params
                });

              case 9:
                response = _context17.sent;

                if (response) {
                  _context17.next = 12;
                  break;
                }

                throw new _ExchangeError.default("Gdax withdraw() error: ".concat(JSON.stringify(response)));

              case 12:
                return _context17.abrupt("return", {
                  info: response,
                  id: response.id
                });

              case 13:
              case "end":
                return _context17.stop();
            }
          }
        }, _callee17, this);
      }));

      return function withdraw() {
        return _withdraw.apply(this, arguments);
      };
    }()
  }]);

  return Gdax;
}(_BaseExchange2.default);

Object.defineProperty(Gdax, "Parser", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _GdaxParser.default
});
Object.defineProperty(Gdax, "REQUIRED_CREDENTIALS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.REQUIRED_CREDENTIALS
});
Object.defineProperty(Gdax, "URLS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.URLS
});
Object.defineProperty(Gdax, "FEES", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.FEES
});
Object.defineProperty(Gdax, "API", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.API
});
Object.defineProperty(Gdax, "SIGNED_APIS", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: _constants2.SIGNED_APIS
});
var _default = Gdax;
exports.default = _default;