"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _omit = _interopRequireDefault(require("lodash/omit"));

var _sortBy = _interopRequireDefault(require("lodash/sortBy"));

var _time = require("../../utils/time");

var _constants = require("../../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var BaseParser =
/*#__PURE__*/
function () {
  function BaseParser() {
    var _this = this;

    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        exchange = _ref.exchange,
        _ref$includeInfo = _ref.includeInfo,
        includeInfo = _ref$includeInfo === void 0 ? false : _ref$includeInfo;

    _classCallCheck(this, BaseParser);

    Object.defineProperty(this, "infoField", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(data) {
        return _this.includeInfo ? {
          info: data
        } : {};
      }
    });
    Object.defineProperty(this, "getCurrencyUsedOnOpenOrders", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(orders, currency) {
        return Object.values(orders).filter(function (order) {
          return order.status === _constants.ORDER_STATUSES.LOWER.OPEN;
        }).reduce(function (total, order) {
          var symbol = order.symbol;
          var market = _this.markets[symbol];
          var amount = order.remaining;

          if (currency === market.base && order.side === _constants.ORDER_SIDES.LOWER.SELL) {
            return total + amount;
          } else if (currency === market.quote && order.side === _constants.ORDER_SIDES.LOWER.BUY) {
            return total + (order.cost || order.price * amount);
          }

          return total;
        }, 0);
      }
    });
    Object.defineProperty(this, "parseBalance", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(balance) {
        var currencies = Object.keys((0, _omit.default)(balance, 'info'));
        var scopedBalance = balance;
        var orders = _this.exchange.orders;
        currencies.forEach(function (currency) {
          if (!balance[currency].used) {
            if (_this.parseBalanceFromOpenOrders && 'open_orders' in balance.info) {
              var OPEN = _constants.ORDER_STATUSES.LOWER.OPEN;
              var exchangeOrdersCount = balance.info.open_orders;
              var orderValues = Object.values(orders);
              var openOrders = orderValues.filter(function (order) {
                return order.status === OPEN;
              });
              var cachedOrdersCount = openOrders.length;

              if (cachedOrdersCount === exchangeOrdersCount) {
                scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.USED] = _this.getCurrencyUsedOnOpenOrders(orders, currency);
                scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.TOTAL] = scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.USED] + scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.FREE];
              }
            } else {
              scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.USED] = _this.getCurrencyUsedOnOpenOrders(orders, currency);
              scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.TOTAL] = scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.USED] + scopedBalance[currency][_constants.BALANCE_TYPES.LOWER.FREE];
            }
          }
        });
        return scopedBalance;
      }
    });
    Object.defineProperty(this, "parseOHLCV", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(ohlcv) {
        return ohlcv;
      }
    });
    Object.defineProperty(this, "parseOHLCVs", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(ohlcvs) {
        var market = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
        var timeframe = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '1m';
        var since = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
        var limit = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;
        var ohlcvsValues = Object.values(ohlcvs);
        var result = [];
        ohlcvsValues.forEach(function (ohlcvsValue) {
          if (limit && result.length >= limit) return null;

          var ohlcv = _this.parseOHLCV(ohlcvsValue, market, timeframe, since, limit);

          if (since && ohlcv[0] < since) return null;
          return result.push(ohlcv);
        });
        return result;
      }
    });
    Object.defineProperty(this, "filterBySinceLimit", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(array) {
        var since = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
        var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;
        var scopedArray = array;

        if (since) {
          scopedArray = array.filter(function (entry) {
            return entry.timestamp > since;
          });
        }

        if (limit) {
          scopedArray = scopedArray.slice(0, limit);
        }

        return scopedArray;
      }
    });
    Object.defineProperty(this, "parseBidAsk", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(bidask) {
        var priceKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
        var amountKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
        var price = parseFloat(bidask[priceKey]);
        var amount = parseFloat(bidask[amountKey]);
        return [price, amount];
      }
    });
    Object.defineProperty(this, "parseBidsAsks", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(bidasks) {
        var priceKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
        var amountKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
        return Object.values(bidasks || []).map(function (bidask) {
          return _this.parseBidAsk(bidask, priceKey, amountKey);
        });
      }
    });
    Object.defineProperty(this, "parseOrderBook", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(orderbook) {
        var timestamp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _time.milliseconds)();
        var bidsKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'bids';
        var asksKey = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'asks';
        var priceKey = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
        var amountKey = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 1;
        return {
          bids: (0, _sortBy.default)(bidsKey in orderbook ? _this.parseBidsAsks(orderbook[bidsKey], priceKey, amountKey) : [], 0, true),
          asks: (0, _sortBy.default)(asksKey in orderbook ? _this.parseBidsAsks(orderbook[asksKey], priceKey, amountKey) : [], 0),
          timestamp: timestamp,
          datetime: (0, _time.iso8601)(timestamp)
        };
      }
    });
    Object.defineProperty(this, "parseTrades", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(trades, market, since, limit) {
        var result = Object.values(trades).map(function (trade) {
          return _this.parseTrade(trade, market);
        });
        result = (0, _sortBy.default)(result, 'timestamp', true);
        return _this.filterBySinceLimit(result, since, limit);
      }
    });
    this.exchange = exchange;
    this.includeInfo = includeInfo;
  }

  _createClass(BaseParser, [{
    key: "parseOrders",
    value: function parseOrders(orders) {
      var _this2 = this;

      var market = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var since = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;
      var limit = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
      var result = Object.values(orders).map(function (order) {
        return _this2.parseOrder(order, market);
      });
      return this.filterBySinceLimit(result, since, limit);
    }
  }]);

  return BaseParser;
}();

var _default = BaseParser;
exports.default = _default;